function f = kernelMult(d,x1,x2,x3)

switch d.name
    case 'rbf'
        if nargin==3
            dot_a = sum(x1.*x1, 1);
            unitvec = ones(1,size(x1,2));
            f = exp(d.sigma * (2 * x1' * x1 -  dot_a - dot_a' * unitvec)) * x2;
        elseif nargin==4
            dot_a = sum(x1.*x1, 1);
            dot_b = sum(x2.*x2, 1);
            unitvec = ones(1,size(x2,2));
            f =exp(d.sigma * (2 * x1' * x2 - ones(size(dot_a,2),1)*dot_b - dot_a' * unitvec)) * x3;
        else
            errormsg('wrong number of input arguments');
        end            
    case 'poly'
        if nargin==3
            f  = (d.scale * x1' * x1 +d.offset).^d.degree * x2;
        elseif nargin==4
            f =(d.scale * x1' * x2 + d.offset).^d.degree * x3;
        else
            errormsg('wrong number of input arguments');
        end            
    case 'polynorm'
        if nargin==3
            f = (((d.scale * x1' * x1 +d.offset).^d.degree)./ ...
                (diag(d.scale * x1' * x1 +1).^d.degree))* x2;
        elseif nargin==4
            f = (((d.scale * x1' * x2 +d.offset).^d.degree)./ ...
                (sqrt(diag((d.scale * x1' * x1 +d.offset).^d.degree)*...
                diag((d.scale * x2' * x2 +d.offset).^d.degree)')))* x3;
        else
            error('wrong number of input arguments');
        end   
    case 'gauss'
        if nargin==3
            dot_a = sum(x1.*x1, 1);
            unitvec = ones(1,size(x1,2));
            f = (1/sqrt(2*pi*d.sigma))*(exp((2 * x1' * x1 -  dot_a - dot_a' * unitvec)./(2*d.sigma)) * x2);
        elseif nargin==4
            dot_a = sum(x1.*x1, 1);
            dot_b = sum(x2.*x2, 1);
            unitvec = ones(1,size(x2,2));
            f = (1/sqrt(2*pi*d.sigma))*(exp((2 * x1' * x2 - ones(size(dot_a,2),1)*dot_b - dot_a' * unitvec)./(2*d.sigma)) * x3);
        else
            errormsg('wrong number of input arguments');
        end         
    case 'epanech'   
        if nargin==3
            dot_a = sum(x1.*x1, 1);
            unitvec = ones(1,size(x1,2));
            f = (max(1+d.sigma * (2 * x1' * x1 -  dot_a - dot_a' * unitvec * x2),0));
        elseif nargin==4
            dot_a = sum(x1.*x1, 1);
            dot_b = sum(x2.*x2, 1);
            unitvec = ones(size(x1,2),1);
            f = x1' * x2;
            for i=1:size(x2,2)
                f(:,i) = (max(1+(d.sigma * (2 * f(:,i) - dot_a' - dot_b(i) * unitvec)),0))* x3(i);
	    end
        else
            errormsg('wrong number of input arguments');
        end

        % Custom kernel added by JP:
    case 'custom'
        if nargin==3
            f = d.K(x1,x1) * x2;
        elseif nargin==4
            f = d.K(x1,x2) * x3;
        else
            error('wrong number of input arguments');
        end

    case 'tppk'
        if nargin==3
            f = ( ( d.K(x1(1,:),x1(1,:)) .* d.K(x1(2,:),x1(2,:)) ) + ( d.K(x1(1,:),x1(2,:)) .* d.K(x1(2,:),x1(1,:)) ) ) * x2;
        elseif nargin==4
            f = ( ( d.K(x1(1,:),x2(1,:)) .* d.K(x1(2,:),x2(2,:)) ) + ( d.K(x1(1,:),x2(2,:)) .* d.K(x1(2,:),x2(1,:)) ) ) * x3;
        else
            error('wrong number of input arguments');
        end

        % end of JP's custom kernel

end