function f = kernelPol(d,x1,x2,x3,x4)


switch d.name
    case 'rbf'
        if nargin==3
            dot_a = sum(x1.*x1, 1);
            unitvec = ones(1,size(x1,2));
            f = x1' * x1;
            for i=1:size(x1,2)
                f(:,i) = x2(i) * (exp(d.sigma * (2 * f(:,i) - dot_a' - dot_a(i) * unitvec)) .* x2);
            end
        elseif nargin==5
            dot_a = sum(x1.*x1,1);
            dot_b = sum(x2.*x2,1);
            unitvec = ones(size(x1,2),1);
            f = x1' * x2;
            for i=1:size(x2,2)
                f(:,i) = x4(i) * (exp(d.sigma * (2 * f(:,i) - dot_a' - dot_b(i) * unitvec)) .* x3);
            end
        else
            error('wrong number of input arguments');
        end
    case 'gauss'
        if nargin==3
            dot_a = sum(x1.*x1, 1);
            unitvec = ones(1,size(x1,2));
            f = x1' * x1;
            for i=1:size(x1,2)
                f(:,i) = (1/sqrt(2*pi*d.sigma))*(x2(i) * (exp((2 * f(:,i) - dot_a' - dot_a(i) * unitvec)./(2*d.sigma)) .* x2));
            end
        elseif nargin==5
            dot_a = sum(x1.*x1,1);
            dot_b = sum(x2.*x2,1);
            unitvec = ones(size(x1,2),1);
            f = x1' * x2;
            for i=1:size(x2,2)
                f(:,i) = (1/sqrt(2*pi*d.sigma))*(x4(i) * (exp((2 * f(:,i) - dot_a' - dot_b(i) * unitvec)./(2*d.sigma)) .* x3));
            end
        else
            error('wrong number of input arguments');
        end                 
    case 'poly'
        if nargin==3
            f = (d.scale * x1' * x1 + d.offset).^d.degree;
            for i=1:size(x1,2)
                f(:,i) = x2(i) * (f(:,i) .* x2);
            end
        elseif nargin==5
            f = (d.scale * x1' * x2 + d.offset).^d.degree;
            for i=1:size(x2,2)
                f(:,i) = x4(i) * (f(:,i) .* x3);
            end
        else
            error('wrong number of input arguments');
        end                   
    case 'polynorm'
        temp.name = 'poly';
        temp.scale = d.scale;
        temp.degree = d.degree;
        if nargin==3
            f = kernelPol(temp, x1, x2)./(sqrt(diag(kernelPol(temp, x1, x2))*diag(kernelPol(temp, x1, x2))'));
        elseif nargin==5
            f = kernelPol(temp, x1, x2, x3, x4)./(sqrt(diag(kernelPol(temp, x1, x3))*diag(kernelPol(temp, x2, x4))'));
        else
            error('wrong number of input arguments');
        end    
    case 'epanech'
        if nargin==3
            dot_a = sum(x1.*x1, 1);
            unitvec = ones(1,size(x1,2));
            f = x1' * x1;
            for i=1:size(x1,2)
                f(:,i) = x2(i) * (max(1+(d.sigma * (2 * f(:,i) - dot_a' - dot_a(i) * unitvec)) .* x2,0));
            end
        elseif nargin==5
            dot_a = sum(x1.*x1,1);
            dot_b = sum(x2.*x2,1);
            unitvec = ones(size(x1,2),1);
            f = x1' * x2;
            for i=1:size(x2,2)
                f(:,i) = x4(i) *(max(1+(d.sigma * (2 * f(:,i) - dot_a' - dot_b(i) * unitvec)),0)).* x3;
            end
        else
            error('wrong number of input arguments');
        end                   
        
        % Custom kernel added by JP:
    case 'custom'
        if nargin==3
            f = d.K(x1,x1) .* (x2*x2');
        elseif nargin==5
            f = d.K(x1,x2) .* (x3*x4');
        else
            error('wrong number of input arguments');
        end

    case 'tppk'
        if nargin==3
            f = ( ( d.K(x1(1,:),x1(1,:)) .* d.K(x1(2,:),x1(2,:)) ) + ( d.K(x1(1,:),x1(2,:)) .* d.K(x1(2,:),x1(1,:)) ) ) .* (x2*x2');
        elseif nargin==5
            f = ( ( d.K(x1(1,:),x2(1,:)) .* d.K(x1(2,:),x2(2,:)) ) + ( d.K(x1(1,:),x2(2,:)) .* d.K(x1(2,:),x2(1,:)) ) ) .* (x3*x4');
        else
            error('wrong number of input arguments');
        end
        
        % end of JP's custom kernel
        
end
